/** @packageDocumentation
 * @module RPC
 */
import { Id64String } from "@bentley/bentleyjs-core";
import { IModelRpcProps, RpcInterface } from "@bentley/imodeljs-common";
import { ContentJSON } from "./content/Content";
import { DescriptorJSON, DescriptorOverrides, SelectionInfo } from "./content/Descriptor";
import { ItemJSON } from "./content/Item";
import { DisplayValueGroupJSON } from "./content/Value";
import { DiagnosticsOptions, DiagnosticsScopeLogs } from "./Diagnostics";
import { InstanceKeyJSON } from "./EC";
import { ElementProperties } from "./ElementProperties";
import { PresentationStatus } from "./Error";
import { NodeKeyJSON } from "./hierarchy/Key";
import { NodeJSON } from "./hierarchy/Node";
import { NodePathElementJSON } from "./hierarchy/NodePathElement";
import { KeySetJSON } from "./KeySet";
import { LabelDefinitionJSON } from "./LabelDefinition";
import { ComputeSelectionRequestOptions, ContentDescriptorRequestOptions, ContentRequestOptions, DisplayLabelRequestOptions, DisplayLabelsRequestOptions, DistinctValuesRequestOptions, ElementPropertiesRequestOptions, ExtendedContentRequestOptions, ExtendedHierarchyRequestOptions, HierarchyCompareOptions, HierarchyRequestOptions, LabelRequestOptions, Paged, SelectionScopeRequestOptions } from "./PresentationManagerOptions";
import { RulesetVariableJSON } from "./RulesetVariables";
import { SelectionScope } from "./selection/SelectionScope";
import { HierarchyCompareInfoJSON, PartialHierarchyModificationJSON } from "./Update";
import { Omit, PagedResponse } from "./Utils";
/**
 * Base options for all presentation RPC requests.
 * @public
 */
export declare type PresentationRpcRequestOptions<TManagerRequestOptions> = Omit<TManagerRequestOptions, "imodel" | "diagnostics"> & {
    /** @internal ID of the client requesting data */
    clientId?: string;
    /** @alpha */
    diagnostics?: DiagnosticsOptions;
};
/**
 * Data structure for presentation RPC responses
 * @public
 */
export declare type PresentationRpcResponse<TResult = undefined> = Promise<{
    /** Response status code */
    statusCode: PresentationStatus;
    /** In case of an error response, the error message */
    errorMessage?: string;
    /** In case of a success response, the result */
    result?: TResult;
    /** @alpha */
    diagnostics?: DiagnosticsScopeLogs[];
}>;
/**
 * Data structure for base hierarchy request options.
 * @public
 * @deprecated Use [[ExtendedHierarchyRpcRequestOptions]]
 */
export declare type HierarchyRpcRequestOptions = PresentationRpcRequestOptions<HierarchyRequestOptions<never, RulesetVariableJSON>>;
/**
 * Data structure for hierarchy request options.
 * @public
 */
export declare type ExtendedHierarchyRpcRequestOptions = PresentationRpcRequestOptions<ExtendedHierarchyRequestOptions<never, NodeKeyJSON, RulesetVariableJSON>>;
/**
 * Data structure for content request options.
 * @public
 * @deprecated Use [[ContentDescriptorRpcRequestOptions]] or [[ExtendedContentRpcRequestOptions]]
 */
export declare type ContentRpcRequestOptions = PresentationRpcRequestOptions<ContentRequestOptions<never, RulesetVariableJSON>>;
/**
 * Data structure for content descriptor RPC request options.
 * @public
 */
export declare type ContentDescriptorRpcRequestOptions = PresentationRpcRequestOptions<ContentDescriptorRequestOptions<never, KeySetJSON, RulesetVariableJSON>>;
/**
 * Data structure for content RPC request options.
 * @public
 */
export declare type ExtendedContentRpcRequestOptions = PresentationRpcRequestOptions<ExtendedContentRequestOptions<never, DescriptorJSON, KeySetJSON, RulesetVariableJSON>>;
/**
 * Data structure for element properties RPC request options.
 * @beta
 */
export declare type ElementPropertiesRpcRequestOptions = PresentationRpcRequestOptions<ElementPropertiesRequestOptions<never>>;
/**
 * Data structure for distinct values' request options.
 * @public
 */
export declare type DistinctValuesRpcRequestOptions = PresentationRpcRequestOptions<DistinctValuesRequestOptions<never, DescriptorJSON, KeySetJSON, RulesetVariableJSON>>;
/**
 * Data structure for label request options.
 * @public
 * @deprecated Use [[DisplayLabelRpcRequestOptions]] or [[DisplayLabelsRpcRequestOptions]]
 */
export declare type LabelRpcRequestOptions = PresentationRpcRequestOptions<LabelRequestOptions<never>>;
/**
 * Data structure for label request options.
 * @public
 */
export declare type DisplayLabelRpcRequestOptions = PresentationRpcRequestOptions<DisplayLabelRequestOptions<never, InstanceKeyJSON>>;
/**
 * Data structure for labels request options.
 * @public
 */
export declare type DisplayLabelsRpcRequestOptions = PresentationRpcRequestOptions<DisplayLabelsRequestOptions<never, InstanceKeyJSON>>;
/**
 * Data structure for selection scope request options.
 * @public
 */
export declare type SelectionScopeRpcRequestOptions = PresentationRpcRequestOptions<SelectionScopeRequestOptions<never>>;
/**
 * @alpha
 */
export declare type ComputeSelectionRpcRequestOptions = PresentationRpcRequestOptions<ComputeSelectionRequestOptions<never>>;
/**
 * Data structure for comparing presentation data after ruleset or ruleset variable changes.
 * @public
 */
export declare type HierarchyCompareRpcOptions = PresentationRpcRequestOptions<HierarchyCompareOptions<never, NodeKeyJSON, RulesetVariableJSON>>;
/**
 * Interface used for communication between Presentation backend and frontend.
 * @public
 */
export declare class PresentationRpcInterface extends RpcInterface {
    /** The immutable name of the interface. */
    static readonly interfaceName = "PresentationRpcInterface";
    /** The semantic version of the interface. */
    static interfaceVersion: string;
    /** @deprecated Use an overload with [[ExtendedHierarchyRpcRequestOptions]] */
    getNodesCount(_token: IModelRpcProps, _options: HierarchyRpcRequestOptions, _parentKey: NodeKeyJSON | undefined): PresentationRpcResponse<number>;
    getNodesCount(_token: IModelRpcProps, _options: ExtendedHierarchyRpcRequestOptions): PresentationRpcResponse<number>;
    /** @deprecated Use [[getPagedNodes]] */
    getNodesAndCount(_token: IModelRpcProps, _options: Paged<HierarchyRpcRequestOptions>, _parentKey?: NodeKeyJSON): PresentationRpcResponse<{
        nodes: NodeJSON[];
        count: number;
    }>;
    /** @deprecated Use [[getPagedNodes]] */
    getNodes(_token: IModelRpcProps, _options: Paged<HierarchyRpcRequestOptions>, _parentKey?: NodeKeyJSON): PresentationRpcResponse<NodeJSON[]>;
    getPagedNodes(_token: IModelRpcProps, _options: Paged<ExtendedHierarchyRpcRequestOptions>): PresentationRpcResponse<PagedResponse<NodeJSON>>;
    getNodePaths(_token: IModelRpcProps, _options: Omit<ExtendedHierarchyRpcRequestOptions, "parentKey">, _paths: InstanceKeyJSON[][], _markedIndex: number): PresentationRpcResponse<NodePathElementJSON[]>;
    getFilteredNodePaths(_token: IModelRpcProps, _options: Omit<ExtendedHierarchyRpcRequestOptions, "parentKey">, _filterText: string): PresentationRpcResponse<NodePathElementJSON[]>;
    /** @alpha @deprecated Will be removed in 3.0 */
    loadHierarchy(_token: IModelRpcProps, _options: HierarchyRpcRequestOptions): PresentationRpcResponse<void>;
    /** @deprecated Use an overload with [[ContentDescriptorRpcRequestOptions]] */
    getContentDescriptor(_token: IModelRpcProps, _options: ContentRpcRequestOptions, _displayType: string, _keys: KeySetJSON, _selection: SelectionInfo | undefined): PresentationRpcResponse<DescriptorJSON | undefined>;
    getContentDescriptor(_token: IModelRpcProps, _options: ContentDescriptorRpcRequestOptions): PresentationRpcResponse<DescriptorJSON | undefined>;
    /** @deprecated Use an overload with [[ExtendedContentRpcRequestOptions]] */
    getContentSetSize(_token: IModelRpcProps, _options: ContentRpcRequestOptions, _descriptorOrOverrides: DescriptorJSON | DescriptorOverrides, _keys: KeySetJSON): PresentationRpcResponse<number>;
    getContentSetSize(_token: IModelRpcProps, _options: ExtendedContentRpcRequestOptions): PresentationRpcResponse<number>;
    /** @deprecated Use [[getPagedContent]] or [[getPagedContentSet]] */
    getContent(_token: IModelRpcProps, _options: ContentRpcRequestOptions, _descriptorOrOverrides: DescriptorJSON | DescriptorOverrides, _keys: KeySetJSON): PresentationRpcResponse<ContentJSON | undefined>;
    /** @deprecated Use [[getPagedContent]] or [[getPagedContentSet]] */
    getContentAndSize(_token: IModelRpcProps, _options: ContentRpcRequestOptions, _descriptorOrOverrides: DescriptorJSON | DescriptorOverrides, _keys: KeySetJSON): PresentationRpcResponse<{
        content?: ContentJSON;
        size: number;
    }>;
    getPagedContent(_token: IModelRpcProps, _options: Paged<ExtendedContentRpcRequestOptions>): PresentationRpcResponse<{
        descriptor: DescriptorJSON;
        contentSet: PagedResponse<ItemJSON>;
    } | undefined>;
    getPagedContentSet(_token: IModelRpcProps, _options: Paged<ExtendedContentRpcRequestOptions>): PresentationRpcResponse<PagedResponse<ItemJSON>>;
    /** @beta */
    getElementProperties(_token: IModelRpcProps, _options: ElementPropertiesRpcRequestOptions): PresentationRpcResponse<ElementProperties>;
    /** @deprecated Use [[getPagedDistinctValues]] */
    getDistinctValues(_token: IModelRpcProps, _options: ContentRpcRequestOptions, _descriptor: DescriptorJSON | DescriptorOverrides, _keys: KeySetJSON, _fieldName: string, _maximumValueCount: number): PresentationRpcResponse<string[]>;
    getPagedDistinctValues(_token: IModelRpcProps, _options: DistinctValuesRpcRequestOptions): PresentationRpcResponse<PagedResponse<DisplayValueGroupJSON>>;
    /** @deprecated Use an overload with [[DisplayLabelRpcRequestOptions]] */
    getDisplayLabelDefinition(_token: IModelRpcProps, _options: LabelRpcRequestOptions, _key: InstanceKeyJSON): PresentationRpcResponse<LabelDefinitionJSON>;
    getDisplayLabelDefinition(_token: IModelRpcProps, _options: DisplayLabelRpcRequestOptions): PresentationRpcResponse<LabelDefinitionJSON>;
    /** @deprecated Use [[getPagedDisplayLabelDefinitions]] */
    getDisplayLabelDefinitions(_token: IModelRpcProps, _options: LabelRpcRequestOptions, _keys: InstanceKeyJSON[]): PresentationRpcResponse<LabelDefinitionJSON[]>;
    getPagedDisplayLabelDefinitions(_token: IModelRpcProps, _options: DisplayLabelsRpcRequestOptions): PresentationRpcResponse<PagedResponse<LabelDefinitionJSON>>;
    getSelectionScopes(_token: IModelRpcProps, _options: SelectionScopeRpcRequestOptions): PresentationRpcResponse<SelectionScope[]>;
    computeSelection(_token: IModelRpcProps, _options: SelectionScopeRpcRequestOptions, _ids: Id64String[], _scopeId: string): PresentationRpcResponse<KeySetJSON>;
    /** @alpha */
    computeSelection(_token: IModelRpcProps, _options: ComputeSelectionRpcRequestOptions): PresentationRpcResponse<KeySetJSON>;
    /** @alpha @deprecated Use [[compareHierarchiesPaged]] */
    compareHierarchies(_token: IModelRpcProps, _options: HierarchyCompareRpcOptions): PresentationRpcResponse<PartialHierarchyModificationJSON[]>;
    compareHierarchiesPaged(_token: IModelRpcProps, _options: HierarchyCompareRpcOptions): PresentationRpcResponse<HierarchyCompareInfoJSON>;
}
/** @alpha */
export declare enum PresentationIpcEvents {
    /**
     * ID of an event that's emitted when backend detects changes in presented data.
     */
    Update = "presentation.onUpdate"
}
//# sourceMappingURL=PresentationRpcInterface.d.ts.map