"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RPC
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcRequestsHandler = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const imodeljs_common_1 = require("@bentley/imodeljs-common");
const Error_1 = require("./Error");
const PresentationRpcInterface_1 = require("./PresentationRpcInterface");
/**
 * RPC requests handler that wraps [[PresentationRpcInterface]] and
 * adds handling for cases when backend needs to be synced with client
 * state.
 *
 * @internal
 */
class RpcRequestsHandler {
    constructor(props) {
        this._maxRequestRepeatCount = 5;
        this.clientId = (props && props.clientId) ? props.clientId : bentleyjs_core_1.Guid.createValue();
    }
    dispose() {
    }
    // eslint-disable-next-line @typescript-eslint/naming-convention
    get rpcClient() { return imodeljs_common_1.RpcManager.getClientForInterface(PresentationRpcInterface_1.PresentationRpcInterface); }
    async requestRepeatedly(func, diagnosticsHandler, repeatCount = 1) {
        let diagnostics;
        let error;
        let shouldRepeat = false;
        try {
            const response = await func();
            diagnostics = response.diagnostics;
            if (response.statusCode === Error_1.PresentationStatus.Success)
                return response.result;
            if (response.statusCode === Error_1.PresentationStatus.BackendTimeout && repeatCount < this._maxRequestRepeatCount)
                shouldRepeat = true;
            else
                error = new Error_1.PresentationError(response.statusCode, response.errorMessage);
        }
        catch (e) {
            error = e;
            if (repeatCount < this._maxRequestRepeatCount)
                shouldRepeat = true;
        }
        finally {
            if (diagnosticsHandler && diagnostics)
                diagnosticsHandler(diagnostics);
        }
        if (shouldRepeat) {
            ++repeatCount;
            return this.requestRepeatedly(func, diagnosticsHandler, repeatCount);
        }
        throw error;
    }
    /**
     * Send the request to backend.
     *
     * If the backend responds with [[PresentationStatus.BackendTimeout]] or there's an RPC-related error,
     * the request is repeated up to `this._maxRequestRepeatCount` times. If we fail to get a valid success or error
     * response from the backend, throw the last encountered error.
     *
     * @internal
     */
    async request(func, options, ...additionalOptions) {
        const { imodel, diagnostics } = options, optionsNoIModel = __rest(options, ["imodel", "diagnostics"]);
        const _a = diagnostics !== null && diagnostics !== void 0 ? diagnostics : {}, { handler: diagnosticsHandler } = _a, diagnosticsOptions = __rest(_a, ["handler"]);
        const rpcOptions = Object.assign(Object.assign({}, optionsNoIModel), { clientId: this.clientId });
        if (diagnostics)
            rpcOptions.diagnostics = diagnosticsOptions;
        const doRequest = async () => func(imodel, rpcOptions, ...additionalOptions);
        return this.requestRepeatedly(doRequest, diagnosticsHandler);
    }
    async getNodesCount(options) {
        return this.request(this.rpcClient.getNodesCount.bind(this.rpcClient), options); // eslint-disable-line deprecation/deprecation
    }
    async getPagedNodes(options) {
        return this.request(this.rpcClient.getPagedNodes.bind(this.rpcClient), options);
    }
    async getNodePaths(options, paths, markedIndex) {
        return this.request(this.rpcClient.getNodePaths.bind(this.rpcClient), options, paths, markedIndex);
    }
    async getFilteredNodePaths(options, filterText) {
        return this.request(this.rpcClient.getFilteredNodePaths.bind(this.rpcClient), options, filterText);
    }
    async getContentDescriptor(options) {
        return this.request(this.rpcClient.getContentDescriptor.bind(this.rpcClient), options); // eslint-disable-line deprecation/deprecation
    }
    async getContentSetSize(options) {
        return this.request(this.rpcClient.getContentSetSize.bind(this.rpcClient), options); // eslint-disable-line deprecation/deprecation
    }
    async getPagedContent(options) {
        return this.request(this.rpcClient.getPagedContent.bind(this.rpcClient), options);
    }
    async getPagedContentSet(options) {
        return this.request(this.rpcClient.getPagedContentSet.bind(this.rpcClient), options);
    }
    async getPagedDistinctValues(options) {
        return this.request(this.rpcClient.getPagedDistinctValues.bind(this.rpcClient), options);
    }
    async getElementProperties(options) {
        return this.request(this.rpcClient.getElementProperties.bind(this.rpcClient), options);
    }
    async getDisplayLabelDefinition(options) {
        return this.request(this.rpcClient.getDisplayLabelDefinition.bind(this.rpcClient), options); // eslint-disable-line deprecation/deprecation
    }
    async getPagedDisplayLabelDefinitions(options) {
        return this.request(this.rpcClient.getPagedDisplayLabelDefinitions.bind(this.rpcClient), options);
    }
    async getSelectionScopes(options) {
        return this.request(this.rpcClient.getSelectionScopes.bind(this.rpcClient), options);
    }
    async computeSelection(options) {
        return this.request(this.rpcClient.computeSelection.bind(this.rpcClient), options);
    }
    async compareHierarchies(options) {
        return this.request(this.rpcClient.compareHierarchies.bind(this.rpcClient), options); // eslint-disable-line deprecation/deprecation
    }
    async compareHierarchiesPaged(options) {
        return this.request(this.rpcClient.compareHierarchiesPaged.bind(this.rpcClient), options);
    }
}
exports.RpcRequestsHandler = RpcRequestsHandler;
//# sourceMappingURL=RpcRequestsHandler.js.map