"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Content
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyOptionalPrefix = exports.FIELD_NAMES_SEPARATOR = exports.addFieldHierarchy = exports.traverseContentItem = exports.traverseContent = exports.traverseFieldHierarchy = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const Fields_1 = require("./Fields");
const Item_1 = require("./Item");
const TypeDescription_1 = require("./TypeDescription");
const Value_1 = require("./Value");
/** @internal */
function traverseFieldHierarchy(hierarchy, cb) {
    if (cb(hierarchy))
        hierarchy.childFields.forEach((childHierarchy) => traverseFieldHierarchy(childHierarchy, cb));
}
exports.traverseFieldHierarchy = traverseFieldHierarchy;
/** @alpha */
function traverseContent(visitor, content) {
    if (!visitor.startContent({ descriptor: content.descriptor }))
        return;
    try {
        const fieldHierarchies = createFieldHierarchies(content.descriptor.fields);
        visitor.processFieldHierarchies({ hierarchies: fieldHierarchies });
        content.contentSet.forEach((item) => {
            traverseContentItemFields(visitor, fieldHierarchies, item);
        });
    }
    finally {
        visitor.finishContent();
    }
}
exports.traverseContent = traverseContent;
/** @alpha */
function traverseContentItem(visitor, descriptor, item) {
    if (!visitor.startContent({ descriptor }))
        return;
    try {
        const fieldHierarchies = createFieldHierarchies(descriptor.fields);
        visitor.processFieldHierarchies({ hierarchies: fieldHierarchies });
        traverseContentItemFields(visitor, fieldHierarchies, item);
    }
    finally {
        visitor.finishContent();
    }
}
exports.traverseContentItem = traverseContentItem;
class VisitedCategories {
    constructor(_visitor, category) {
        this._visitor = _visitor;
        const stack = [];
        let curr = category;
        while (curr) {
            stack.push(curr);
            curr = curr.parent;
        }
        stack.reverse();
        this._didVisitAllHierarchy = true;
        this._visitedCategories = [];
        for (curr of stack) {
            if (this._visitor.startCategory({ category: curr })) {
                this._visitedCategories.push(curr);
            }
            else {
                this._didVisitAllHierarchy = false;
                break;
            }
        }
    }
    dispose() {
        while (this._visitedCategories.pop())
            this._visitor.finishCategory();
    }
    get shouldContinue() { return this._didVisitAllHierarchy; }
}
function traverseContentItemFields(visitor, fieldHierarchies, item) {
    if (!visitor.startItem({ item }))
        return;
    try {
        fieldHierarchies.forEach((fieldHierarchy) => {
            bentleyjs_core_1.using(new VisitedCategories(visitor, fieldHierarchy.field.category), (res) => {
                if (res.shouldContinue)
                    traverseContentItemField(visitor, fieldHierarchy, item);
            });
        });
    }
    finally {
        visitor.finishItem();
    }
}
function traverseContentItemField(visitor, fieldHierarchy, item) {
    if (!visitor.startField({ hierarchy: fieldHierarchy }))
        return;
    try {
        const rootToThisField = createFieldPath(fieldHierarchy.field);
        let namePrefix;
        const pathUpToField = rootToThisField.slice(undefined, -1);
        for (let i = 0; i < pathUpToField.length; ++i) {
            const parentField = pathUpToField[i];
            const nextField = rootToThisField[i + 1];
            if (item.isFieldMerged(parentField.name)) {
                visitor.processMergedValue({ requestedField: fieldHierarchy.field, mergedField: parentField, namePrefix });
                return;
            }
            item = convertNestedContentItemToStructArrayItem(item, parentField, nextField);
            namePrefix = applyOptionalPrefix(parentField.name, namePrefix);
        }
        if (item.isFieldMerged(fieldHierarchy.field.name)) {
            visitor.processMergedValue({ requestedField: fieldHierarchy.field, mergedField: fieldHierarchy.field, namePrefix });
            return;
        }
        if (fieldHierarchy.field.isNestedContentField()) {
            fieldHierarchy = convertNestedContentFieldHierarchyToStructArrayHierarchy(fieldHierarchy, namePrefix);
            item = convertNestedContentFieldHierarchyItemToStructArrayItem(item, fieldHierarchy);
        }
        else if (pathUpToField.length > 0) {
            fieldHierarchy = Object.assign(Object.assign({}, fieldHierarchy), { field: Object.assign(fieldHierarchy.field.clone(), {
                    type: {
                        valueFormat: TypeDescription_1.PropertyValueFormat.Array,
                        typeName: `${fieldHierarchy.field.type.typeName}[]`,
                        memberType: fieldHierarchy.field.type,
                    },
                }) });
        }
        traverseContentItemFieldValue(visitor, fieldHierarchy, item.mergedFieldNames, fieldHierarchy.field.type, namePrefix, item.values[fieldHierarchy.field.name], item.displayValues[fieldHierarchy.field.name]);
    }
    finally {
        visitor.finishField();
    }
}
function traverseContentItemFieldValue(visitor, fieldHierarchy, mergedFieldNames, valueType, namePrefix, rawValue, displayValue) {
    if (rawValue !== undefined) {
        if (valueType.valueFormat === TypeDescription_1.PropertyValueFormat.Array) {
            bentleyjs_core_1.assert(Value_1.Value.isArray(rawValue));
            bentleyjs_core_1.assert(Value_1.DisplayValue.isArray(displayValue));
            return traverseContentItemArrayFieldValue(visitor, fieldHierarchy, mergedFieldNames, valueType, namePrefix, rawValue, displayValue);
        }
        if (valueType.valueFormat === TypeDescription_1.PropertyValueFormat.Struct) {
            bentleyjs_core_1.assert(Value_1.Value.isMap(rawValue));
            bentleyjs_core_1.assert(Value_1.DisplayValue.isMap(displayValue));
            return traverseContentItemStructFieldValue(visitor, fieldHierarchy, mergedFieldNames, valueType, namePrefix, rawValue, displayValue);
        }
    }
    traverseContentItemPrimitiveFieldValue(visitor, fieldHierarchy, mergedFieldNames, valueType, namePrefix, rawValue, displayValue);
}
function traverseContentItemArrayFieldValue(visitor, fieldHierarchy, mergedFieldNames, valueType, namePrefix, rawValues, displayValues) {
    bentleyjs_core_1.assert(rawValues.length === displayValues.length);
    bentleyjs_core_1.assert(valueType.valueFormat === TypeDescription_1.PropertyValueFormat.Array);
    if (!visitor.startArray({ hierarchy: fieldHierarchy, valueType, namePrefix, rawValues, displayValues }))
        return;
    try {
        const itemType = valueType.memberType;
        rawValues.forEach((_, i) => {
            traverseContentItemFieldValue(visitor, fieldHierarchy, mergedFieldNames, itemType, namePrefix, rawValues[i], displayValues[i]);
        });
    }
    finally {
        visitor.finishArray();
    }
}
function traverseContentItemStructFieldValue(visitor, fieldHierarchy, mergedFieldNames, valueType, namePrefix, rawValues, displayValues) {
    bentleyjs_core_1.assert(valueType.valueFormat === TypeDescription_1.PropertyValueFormat.Struct);
    if (!visitor.startStruct({ hierarchy: fieldHierarchy, valueType, namePrefix, rawValues, displayValues }))
        return;
    try {
        if (fieldHierarchy.field.isNestedContentField())
            namePrefix = applyOptionalPrefix(fieldHierarchy.field.name, namePrefix);
        valueType.members.forEach((memberDescription) => {
            let memberField = fieldHierarchy.childFields.find((f) => f.field.name === memberDescription.name);
            if (!memberField) {
                // Not finding a member field means we're traversing an ECStruct. We still need to carry member information, so we
                // create a fake field to represent the member
                memberField = {
                    field: new Fields_1.Field(fieldHierarchy.field.category, memberDescription.name, memberDescription.label, memberDescription.type, fieldHierarchy.field.isReadonly, 0),
                    childFields: [],
                };
            }
            traverseContentItemFieldValue(visitor, memberField, mergedFieldNames, memberDescription.type, namePrefix, rawValues[memberDescription.name], displayValues[memberDescription.name]);
        });
    }
    finally {
        visitor.finishStruct();
    }
}
function traverseContentItemPrimitiveFieldValue(visitor, fieldHierarchy, mergedFieldNames, valueType, namePrefix, rawValue, displayValue) {
    if (-1 !== mergedFieldNames.indexOf(fieldHierarchy.field.name)) {
        visitor.processMergedValue({ mergedField: fieldHierarchy.field, requestedField: fieldHierarchy.field, namePrefix });
        return;
    }
    visitor.processPrimitiveValue({ field: fieldHierarchy.field, valueType, namePrefix, rawValue, displayValue });
}
function createFieldHierarchies(fields) {
    const hierarchies = new Array();
    const visitField = (category, field, parentField) => {
        let childFields = [];
        if (field.isNestedContentField()) {
            // visit all nested fields
            childFields = visitFields(field.nestedFields, field);
            if (0 === childFields.length)
                return undefined;
        }
        const fieldHierarchy = { field, childFields };
        if (category === (parentField === null || parentField === void 0 ? void 0 : parentField.category)) {
            // if categories of this field and its parent field match - return the field hierarchy without
            // including it as a top level field
            return fieldHierarchy;
        }
        addFieldHierarchy(hierarchies, fieldHierarchy);
        return undefined;
    };
    const visitFields = (visitedFields, parentField) => {
        const includedFields = [];
        visitedFields.forEach((field) => {
            const visitedField = visitField(field.category, field, parentField);
            if (visitedField)
                includedFields.push(visitedField);
        });
        return includedFields;
    };
    visitFields(fields, undefined);
    return hierarchies;
}
function findRelatedFields(rootFields, hierarchy) {
    // build a list of parent fields in hierarchy
    const fields = [];
    let currField = hierarchy.field;
    while (currField) {
        fields.push(currField);
        currField = currField.parent;
    }
    for (let rootIndex = 0; rootIndex < rootFields.length; ++rootIndex) {
        const rootFieldHierarchy = rootFields[rootIndex];
        if (rootFieldHierarchy.field.category !== hierarchy.field.category) {
            // only interested in fields with the same category
            continue;
        }
        let first = true;
        currField = rootFieldHierarchy.field;
        while (currField) {
            const index = fields.findIndex((f) => f.name === currField.name);
            if (-1 !== index) {
                return {
                    existing: {
                        field: currField,
                        hierarchy: first ? rootFieldHierarchy : undefined,
                        index: rootIndex,
                    },
                    matchingField: fields[index],
                };
            }
            currField = currField.parent;
            first = false;
        }
    }
    return undefined;
}
function buildParentHierarchy(hierarchy, parentField) {
    // note: parentField is found by walking up the parentship relationship
    // from hierarchy.field, so we expect to always find it here
    while (hierarchy.field !== parentField) {
        const hierarchyParent = hierarchy.field.parent;
        bentleyjs_core_1.assert(hierarchyParent !== undefined);
        hierarchy = { field: hierarchyParent, childFields: [hierarchy] };
    }
    return hierarchy;
}
function mergeHierarchies(lhs, rhs) {
    bentleyjs_core_1.assert(lhs.field.name === rhs.field.name);
    const result = {
        field: lhs.field.clone(),
        childFields: [...lhs.childFields],
    };
    rhs.childFields.forEach((rhsChildHierarchy) => {
        const indexInResult = result.childFields.findIndex((resultHierarchy) => resultHierarchy.field.name === rhsChildHierarchy.field.name);
        if (indexInResult !== -1)
            result.childFields[indexInResult] = mergeHierarchies(result.childFields[indexInResult], rhsChildHierarchy);
        else
            result.childFields.push(rhsChildHierarchy);
    });
    return result;
}
/** @alpha */
function addFieldHierarchy(rootHierarchies, hierarchy) {
    var _a;
    const match = findRelatedFields(rootHierarchies, hierarchy);
    if (!match) {
        rootHierarchies.push(hierarchy);
        return;
    }
    const targetHierarchy = rootHierarchies[match.existing.index];
    const existingHierarchy = (_a = match.existing.hierarchy) !== null && _a !== void 0 ? _a : buildParentHierarchy(targetHierarchy, match.existing.field);
    const insertHierarchy = buildParentHierarchy(hierarchy, match.matchingField);
    const mergedHierarchy = mergeHierarchies(existingHierarchy, insertHierarchy);
    mergedHierarchy.field.category = hierarchy.field.category;
    rootHierarchies[match.existing.index] = mergedHierarchy;
}
exports.addFieldHierarchy = addFieldHierarchy;
function createFieldPath(field) {
    const path = [field];
    let currField = field;
    while (currField.parent) {
        currField = currField.parent;
        path.push(currField);
    }
    path.reverse();
    return path;
}
/** @internal */
exports.FIELD_NAMES_SEPARATOR = "$";
/** @internal */
function applyOptionalPrefix(str, prefix) {
    return prefix ? `${prefix}${exports.FIELD_NAMES_SEPARATOR}${str}` : str;
}
exports.applyOptionalPrefix = applyOptionalPrefix;
function convertNestedContentItemToStructArrayItem(item, field, nextField) {
    var _a;
    const value = (_a = item.values[field.name]) !== null && _a !== void 0 ? _a : [];
    bentleyjs_core_1.assert(Value_1.Value.isNestedContent(value));
    const nextFieldValues = { raw: [], display: [] };
    value.forEach((ncv) => {
        const nextRawValue = ncv.values[nextField.name];
        const nextDisplayValue = ncv.displayValues[nextField.name];
        if (nextField.isNestedContentField()) {
            if (nextRawValue) {
                bentleyjs_core_1.assert(Value_1.Value.isNestedContent(nextRawValue));
                nextFieldValues.raw.push(...nextRawValue);
            }
        }
        else {
            nextFieldValues.raw.push(nextRawValue);
            nextFieldValues.display.push(nextDisplayValue);
        }
    });
    return new Item_1.Item(item.primaryKeys, item.label, item.imageId, item.classInfo, { [nextField.name]: nextFieldValues.raw }, { [nextField.name]: nextFieldValues.display }, item.mergedFieldNames, item.extendedData);
}
function convertNestedContentFieldHierarchyToStructArrayHierarchy(fieldHierarchy, namePrefix) {
    const fieldName = fieldHierarchy.field.name;
    const convertedChildFieldHierarchies = fieldHierarchy.childFields.map((child) => {
        if (child.field.isNestedContentField())
            return convertNestedContentFieldHierarchyToStructArrayHierarchy(child, applyOptionalPrefix(fieldName, namePrefix));
        return child;
    });
    const convertedFieldHierarchy = {
        field: Object.assign(fieldHierarchy.field.clone(), {
            type: {
                valueFormat: TypeDescription_1.PropertyValueFormat.Array,
                typeName: `${fieldHierarchy.field.type.typeName}[]`,
                memberType: {
                    valueFormat: TypeDescription_1.PropertyValueFormat.Struct,
                    typeName: fieldHierarchy.field.type.typeName,
                    members: convertedChildFieldHierarchies.map((member) => ({
                        name: member.field.name,
                        label: member.field.label,
                        type: member.field.type,
                    })),
                },
            },
        }),
        childFields: convertedChildFieldHierarchies,
    };
    return convertedFieldHierarchy;
}
function convertNestedContentValuesToStructArrayValuesRecursive(fieldHierarchy, ncvs) {
    const result = { raw: [], display: [], mergedFieldNames: [] };
    ncvs.forEach((ncv) => {
        const values = Object.assign({}, ncv.values);
        const displayValues = Object.assign({}, ncv.displayValues);
        const mergedFieldNames = [...ncv.mergedFieldNames];
        fieldHierarchy.childFields.forEach((childFieldHierarchy) => {
            const childFieldName = childFieldHierarchy.field.name;
            if (-1 !== ncv.mergedFieldNames.indexOf(childFieldName)) {
                return;
            }
            if (childFieldHierarchy.field.isNestedContentField()) {
                const value = values[childFieldName];
                bentleyjs_core_1.assert(Value_1.Value.isNestedContent(value));
                const convertedValues = convertNestedContentValuesToStructArrayValuesRecursive(childFieldHierarchy, value);
                values[childFieldName] = convertedValues.raw;
                displayValues[childFieldName] = convertedValues.display;
                mergedFieldNames.push(...convertedValues.mergedFieldNames);
            }
        });
        result.raw.push(values);
        result.display.push(displayValues);
        result.mergedFieldNames.push(...mergedFieldNames);
    });
    return result;
}
function convertNestedContentFieldHierarchyItemToStructArrayItem(item, fieldHierarchy) {
    const fieldName = fieldHierarchy.field.name;
    const rawValue = item.values[fieldName];
    bentleyjs_core_1.assert(Value_1.Value.isNestedContent(rawValue));
    const converted = convertNestedContentValuesToStructArrayValuesRecursive(fieldHierarchy, rawValue);
    return new Item_1.Item(item.primaryKeys, item.label, item.imageId, item.classInfo, { [fieldName]: converted.raw }, { [fieldName]: converted.display }, converted.mergedFieldNames, item.extendedData);
}
//# sourceMappingURL=ContentTraverser.js.map