"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Content
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FieldDescriptor = exports.FieldDescriptorType = exports.getFieldByName = exports.NestedContentField = exports.PropertiesField = exports.Field = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const EC_1 = require("../EC");
const Error_1 = require("../Error");
const RelatedPropertiesSpecification_1 = require("../rules/content/modifiers/RelatedPropertiesSpecification");
const Category_1 = require("./Category");
const Property_1 = require("./Property");
/** Is supplied field a properties field. */
const isPropertiesField = (field) => {
    return !!field.properties;
};
/** Is supplied field a nested content field. */
const isNestedContentField = (field) => {
    return !!field.nestedFields;
};
/**
 * Describes a single content field. A field is usually represented as a grid column
 * or a property pane row.
 *
 * @public
 */
class Field {
    /**
     * Creates an instance of Field.
     * @param category Category information
     * @param name Unique name
     * @param label Display label
     * @param type Description of this field's values data type
     * @param isReadonly Are values in this field read-only
     * @param priority Priority of the field
     * @param editor Property editor used to edit values of this field
     * @param renderer Property renderer used to render values of this field
     */
    constructor(category, name, label, type, isReadonly, priority, editor, renderer) {
        this.category = category;
        this.name = name;
        this.label = label;
        this.type = type;
        this.isReadonly = isReadonly;
        this.priority = priority;
        this.editor = editor;
        this.renderer = renderer;
    }
    /**
     * Is this a [[PropertiesField]]
     */
    isPropertiesField() { return isPropertiesField(this); }
    /**
     * Is this a [[NestedContentField]]
     */
    isNestedContentField() { return isNestedContentField(this); }
    /**
     * Get parent
     */
    get parent() { return this._parent; }
    clone() {
        const clone = new Field(this.category, this.name, this.label, this.type, this.isReadonly, this.priority, this.editor, this.renderer);
        clone.rebuildParentship(this.parent);
        return clone;
    }
    /** Serialize this object to JSON */
    toJSON() {
        return {
            category: Category_1.CategoryDescription.toJSON(this.category),
            name: this.name,
            label: this.label,
            type: this.type,
            isReadonly: this.isReadonly,
            priority: this.priority,
            renderer: this.renderer,
            editor: this.editor,
        };
    }
    /** Serialize this object to compressed JSON */
    toCompressedJSON(classesMap) {
        if (this.isPropertiesField())
            return Object.assign(Object.assign({}, this.toJSON()), { properties: this.properties.map((property) => Property_1.Property.toCompressedJSON(property, classesMap)) });
        if (this.isNestedContentField()) {
            const _a = this.contentClassInfo, { id } = _a, leftOverInfo = __rest(_a, ["id"]);
            classesMap[id] = leftOverInfo;
            return Object.assign(Object.assign({}, this.toJSON()), { contentClassInfo: id, pathToPrimaryClass: this.pathToPrimaryClass.map((classInfo) => EC_1.RelatedClassInfo.toCompressedJSON(classInfo, classesMap)), nestedFields: this.nestedFields.map((field) => field.toCompressedJSON(classesMap)) });
        }
        return this.toJSON();
    }
    static fromJSON(json, categories) {
        if (!json)
            return undefined;
        if (typeof json === "string") {
            // eslint-disable-next-line deprecation/deprecation
            return JSON.parse(json, Field.reviver);
        }
        if (isPropertiesField(json))
            return PropertiesField.fromJSON(json, categories);
        if (isNestedContentField(json))
            return NestedContentField.fromJSON(json, categories);
        const field = Object.create(Field.prototype);
        return Object.assign(field, json, {
            category: Field.getCategoryFromFieldJson(json, categories),
        });
    }
    /** Deserialize [[Field]] from compressed JSON */
    static fromCompressedJSON(json, classesMap) {
        if (isPropertiesField(json))
            return fromCompressedPropertiesFieldJSON(json, classesMap);
        if (isNestedContentField(json))
            return fromCompressedNestedContentFieldJSON(json, classesMap);
        return json;
    }
    static getCategoryFromFieldJson(fieldJson, categories) {
        const category = categories ? categories.find((c) => c.name === ((typeof fieldJson.category === "string") ? fieldJson.category : fieldJson.category.name))
            : (typeof fieldJson.category === "object") ? Category_1.CategoryDescription.fromJSON(fieldJson.category) : undefined;
        if (!category)
            throw new Error_1.PresentationError(Error_1.PresentationStatus.InvalidArgument, `Invalid content field category`);
        return category;
    }
    /**
     * Reviver function that can be used as a second argument for
     * `JSON.parse` method when parsing Field objects.
     *
     * @internal
     * @deprecated Use [[fromJSON]]
     */
    static reviver(key, value) {
        // eslint-disable-next-line deprecation/deprecation
        return key === "" ? Field.fromJSON(value) : value;
    }
    /** @internal */
    resetParentship() {
        this._parent = undefined;
    }
    /** @internal */
    rebuildParentship(parentField) {
        this._parent = parentField;
    }
    /**
     * Get descriptor for this field.
     * @public
     */
    getFieldDescriptor() {
        return {
            type: FieldDescriptorType.Name,
            fieldName: this.name,
        };
    }
}
exports.Field = Field;
/**
 * Describes a content field that's based on one or more similar
 * EC properties.
 *
 * @public
 */
class PropertiesField extends Field {
    /**
     * Creates an instance of PropertiesField.
     * @param category Category information
     * @param name Unique name
     * @param label Display label
     * @param type Description of this field's values data type
     * @param isReadonly Are values in this field read-only
     * @param priority Priority of the field
     * @param properties A list of properties this field is created from
     * @param editor Property editor used to edit values of this field
     * @param renderer Property renderer used to render values of this field
     */
    constructor(category, name, label, description, isReadonly, priority, properties, editor, renderer) {
        super(category, name, label, description, isReadonly, priority, editor, renderer);
        this.properties = properties;
    }
    clone() {
        const clone = new PropertiesField(this.category, this.name, this.label, this.type, this.isReadonly, this.priority, this.properties, this.editor, this.renderer);
        clone.rebuildParentship(this.parent);
        return clone;
    }
    /** Serialize this object to JSON */
    toJSON() {
        return Object.assign(Object.assign({}, super.toJSON()), { properties: this.properties.map((p) => Property_1.Property.toJSON(p)) });
    }
    static fromJSON(json, categories) {
        if (!json)
            return undefined;
        if (typeof json === "string") {
            // eslint-disable-next-line deprecation/deprecation
            return JSON.parse(json, Field.reviver);
        }
        const field = Object.create(PropertiesField.prototype);
        return Object.assign(field, json, {
            category: this.getCategoryFromFieldJson(json, categories),
            properties: json.properties.map(Property_1.Property.fromJSON),
        });
    }
    /**
     * Get descriptor for this field.
     * @public
     */
    getFieldDescriptor() {
        const pathFromPropertyToSelectClass = new Array();
        let currAncestor = this.parent;
        while (currAncestor) {
            pathFromPropertyToSelectClass.push(...currAncestor.pathToPrimaryClass);
            currAncestor = currAncestor.parent;
        }
        return {
            type: FieldDescriptorType.Properties,
            pathFromSelectToPropertyClass: EC_1.RelationshipPath.strip(EC_1.RelationshipPath.reverse(pathFromPropertyToSelectClass)),
            properties: this.properties.map((p) => ({
                class: p.property.classInfo.name,
                name: p.property.name,
            })),
        };
    }
}
exports.PropertiesField = PropertiesField;
/**
 * Describes a content field that contains [Nested content]($docs/learning/presentation/Content/Terminology#nested-content).
 *
 * @public
 */
class NestedContentField extends Field {
    /**
     * Creates an instance of NestedContentField.
     * @param category Category information
     * @param name Unique name
     * @param label Display label
     * @param type Description of this field's values data type
     * @param isReadonly Are values in this field read-only
     * @param priority Priority of the field
     * @param contentClassInfo Information about an ECClass whose properties are nested inside this field
     * @param pathToPrimaryClass Relationship path to [Primary class]($docs/learning/presentation/Content/Terminology#primary-class)
     * @param nestedFields Contained nested fields
     * @param editor Property editor used to edit values of this field
     * @param autoExpand Flag specifying whether field should be expanded
     * @param relationshipMeaning RelationshipMeaning of the field
     * @param renderer Property renderer used to render values of this field
     */
    constructor(category, name, label, description, isReadonly, priority, contentClassInfo, pathToPrimaryClass, nestedFields, editor, autoExpand, renderer) {
        super(category, name, label, description, isReadonly, priority, editor, renderer);
        this.contentClassInfo = contentClassInfo;
        this.pathToPrimaryClass = pathToPrimaryClass;
        this.relationshipMeaning = RelatedPropertiesSpecification_1.RelationshipMeaning.RelatedInstance;
        this.nestedFields = nestedFields;
        this.autoExpand = autoExpand;
        this.actualPrimaryClassIds = [];
    }
    clone() {
        const clone = new NestedContentField(this.category, this.name, this.label, this.type, this.isReadonly, this.priority, this.contentClassInfo, this.pathToPrimaryClass, this.nestedFields.map((n) => n.clone()), this.editor, this.autoExpand, this.renderer);
        clone.actualPrimaryClassIds = this.actualPrimaryClassIds;
        clone.relationshipMeaning = this.relationshipMeaning;
        clone.rebuildParentship(this.parent);
        return clone;
    }
    /**
     * Get field by its name
     * @param name Name of the field to find
     * @param recurse Recurse into nested fields
     */
    getFieldByName(name, recurse) {
        return exports.getFieldByName(this.nestedFields, name, recurse);
    }
    /** Serialize this object to JSON */
    toJSON() {
        return Object.assign(Object.assign({}, super.toJSON()), { contentClassInfo: this.contentClassInfo, pathToPrimaryClass: this.pathToPrimaryClass, relationshipMeaning: this.relationshipMeaning, actualPrimaryClassIds: this.actualPrimaryClassIds, nestedFields: this.nestedFields.map((field) => field.toJSON()), autoExpand: this.autoExpand });
    }
    static fromJSON(json, categories) {
        var _a, _b;
        if (!json)
            return undefined;
        if (typeof json === "string") {
            // eslint-disable-next-line deprecation/deprecation
            return JSON.parse(json, Field.reviver);
        }
        const field = Object.create(NestedContentField.prototype);
        return Object.assign(field, json, {
            category: this.getCategoryFromFieldJson(json, categories),
            nestedFields: json.nestedFields.map((nestedFieldJson) => Field.fromJSON(nestedFieldJson, categories))
                .filter((nestedField) => !!nestedField),
            contentClassInfo: EC_1.ClassInfo.fromJSON(json.contentClassInfo),
            pathToPrimaryClass: json.pathToPrimaryClass.map(EC_1.RelatedClassInfo.fromJSON),
            relationshipMeaning: (_a = json.relationshipMeaning) !== null && _a !== void 0 ? _a : RelatedPropertiesSpecification_1.RelationshipMeaning.RelatedInstance,
            actualPrimaryClassIds: (_b = json.actualPrimaryClassIds) !== null && _b !== void 0 ? _b : [],
            autoExpand: json.autoExpand,
        });
    }
    /** @internal */
    resetParentship() {
        super.resetParentship();
        for (const nestedField of this.nestedFields)
            nestedField.resetParentship();
    }
    /** @internal */
    rebuildParentship(parentField) {
        super.rebuildParentship(parentField);
        for (const nestedField of this.nestedFields)
            nestedField.rebuildParentship(this);
    }
}
exports.NestedContentField = NestedContentField;
/** @internal */
const getFieldByName = (fields, name, recurse) => {
    for (const field of fields) {
        if (field.name === name)
            return field;
        if (recurse && field.isNestedContentField()) {
            const nested = exports.getFieldByName(field.nestedFields, name, recurse);
            if (nested)
                return nested;
        }
    }
    return undefined;
};
exports.getFieldByName = getFieldByName;
/**
 * Types of different field descriptors.
 * @public
 */
var FieldDescriptorType;
(function (FieldDescriptorType) {
    FieldDescriptorType["Name"] = "name";
    FieldDescriptorType["Properties"] = "properties";
})(FieldDescriptorType = exports.FieldDescriptorType || (exports.FieldDescriptorType = {}));
/** @public */
var FieldDescriptor;
(function (FieldDescriptor) {
    /** Is this a named field descriptor */
    function isNamed(d) {
        return d.type === FieldDescriptorType.Name;
    }
    FieldDescriptor.isNamed = isNamed;
    /** Is this a properties field descriptor */
    function isProperties(d) {
        return d.type === FieldDescriptorType.Properties;
    }
    FieldDescriptor.isProperties = isProperties;
})(FieldDescriptor = exports.FieldDescriptor || (exports.FieldDescriptor = {}));
function fromCompressedNestedContentFieldJSON(compressedNestedContentFieldJSON, classesMap) {
    bentleyjs_core_1.assert(classesMap.hasOwnProperty(compressedNestedContentFieldJSON.contentClassInfo));
    return Object.assign(Object.assign({}, compressedNestedContentFieldJSON), { contentClassInfo: Object.assign({ id: compressedNestedContentFieldJSON.contentClassInfo }, classesMap[compressedNestedContentFieldJSON.contentClassInfo]), nestedFields: compressedNestedContentFieldJSON.nestedFields.map((compressedFieldJSON) => Field.fromCompressedJSON(compressedFieldJSON, classesMap)).filter((fromCompressededJson) => !!fromCompressededJson), pathToPrimaryClass: compressedNestedContentFieldJSON.pathToPrimaryClass.map((compressedInfoJSON) => EC_1.RelatedClassInfo.fromCompressedJSON(compressedInfoJSON, classesMap)) });
}
function fromCompressedPropertiesFieldJSON(compressedPropertiesFieldJSON, classesMap) {
    return Object.assign(Object.assign({}, compressedPropertiesFieldJSON), { properties: compressedPropertiesFieldJSON.properties.map((compressedPropertyJSON) => fromCompressedPropertyJSON(compressedPropertyJSON, classesMap)) });
}
function fromCompressedPropertyJSON(compressedPropertyJSON, classesMap) {
    return {
        property: fromCompressedPropertyInfoJSON(compressedPropertyJSON.property, classesMap),
        // eslint-disable-next-line deprecation/deprecation
        relatedClassPath: compressedPropertyJSON.relatedClassPath.map((compressedInfoJSON) => EC_1.RelatedClassInfo.fromCompressedJSON(compressedInfoJSON, classesMap)),
    };
}
function fromCompressedPropertyInfoJSON(compressedPropertyJSON, classesMap) {
    bentleyjs_core_1.assert(classesMap.hasOwnProperty(compressedPropertyJSON.classInfo));
    return Object.assign(Object.assign({}, compressedPropertyJSON), { classInfo: Object.assign({ id: compressedPropertyJSON.classInfo }, classesMap[compressedPropertyJSON.classInfo]) });
}
//# sourceMappingURL=Fields.js.map