"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Content
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DisplayValueGroup = exports.NestedContentValue = exports.DisplayValue = exports.Value = void 0;
const EC_1 = require("../EC");
/** @public */
var Value;
(function (Value) {
    /** Is the value a primitive */
    function isPrimitive(value) { return isPrimitiveValue(value); }
    Value.isPrimitive = isPrimitive;
    /** Is the value an array */
    function isArray(value) { return isArrayValue(value); }
    Value.isArray = isArray;
    /** Is the value a map / struct */
    function isMap(value) { return isMapValue(value); }
    Value.isMap = isMap;
    /** Is the value a nested content value */
    function isNestedContent(value) { return isNestedContentValue(value); }
    Value.isNestedContent = isNestedContent;
    /** Serialize [[Value]] to JSON */
    function fromJSON(json) {
        if (json === null)
            return undefined;
        if (isNestedContentValue(json))
            return json.map(NestedContentValue.fromJSON);
        if (isArrayValue(json))
            return valuesArrayFromJSON(json);
        if (isMapValue(json))
            return valuesMapFromJSON(json);
        return json;
    }
    Value.fromJSON = fromJSON;
    /** Deserialize [[Value]] from JSON */
    function toJSON(value) {
        if (undefined === value)
            return null;
        if (isNestedContentValue(value))
            return value.map(NestedContentValue.toJSON);
        if (isArrayValue(value))
            return valuesArrayToJSON(value);
        if (isMapValue(value))
            return valuesMapToJSON(value);
        return value;
    }
    Value.toJSON = toJSON;
})(Value = exports.Value || (exports.Value = {}));
/** @public */
var DisplayValue;
(function (DisplayValue) {
    /** Is the value a primitive */
    function isPrimitive(value) { return isPrimitiveValue(value); }
    DisplayValue.isPrimitive = isPrimitive;
    /** Is the value an array */
    function isArray(value) { return isArrayValue(value); }
    DisplayValue.isArray = isArray;
    /** Is the value a map / struct */
    function isMap(value) { return isMapValue(value); }
    DisplayValue.isMap = isMap;
    /** Serialize [[DisplayValue]] to JSON */
    function fromJSON(json) {
        if (json === null)
            return undefined;
        if (isArrayValue(json))
            return displayValuesArrayFromJSON(json);
        if (isMapValue(json))
            return displayValuesMapFromJSON(json);
        return json;
    }
    DisplayValue.fromJSON = fromJSON;
    /** Deserialize [[DisplayValue]] from JSON */
    function toJSON(value) {
        if (undefined === value)
            return null;
        if (isArrayValue(value))
            return displayValuesArrayToJSON(value);
        if (isMapValue(value))
            return displayValuesMapToJSON(value);
        return value;
    }
    DisplayValue.toJSON = toJSON;
})(DisplayValue = exports.DisplayValue || (exports.DisplayValue = {}));
/** @public */
var NestedContentValue;
(function (NestedContentValue) {
    /** Serialize [[NestedContentValue]] to JSON */
    function toJSON(json) {
        return {
            primaryKeys: json.primaryKeys.map(EC_1.InstanceKey.toJSON),
            values: valuesMapToJSON(json.values),
            displayValues: displayValuesMapToJSON(json.displayValues),
            mergedFieldNames: json.mergedFieldNames,
        };
    }
    NestedContentValue.toJSON = toJSON;
    /** Deserialize [[NestedContentValue]] from JSON */
    function fromJSON(json) {
        return {
            primaryKeys: json.primaryKeys.map(EC_1.InstanceKey.fromJSON),
            values: valuesMapFromJSON(json.values),
            displayValues: displayValuesMapFromJSON(json.displayValues),
            mergedFieldNames: json.mergedFieldNames,
        };
    }
    NestedContentValue.fromJSON = fromJSON;
})(NestedContentValue = exports.NestedContentValue || (exports.NestedContentValue = {}));
/** @public */
var DisplayValueGroup;
(function (DisplayValueGroup) {
    /** Serialize [[DisplayValueGroup]] to JSON */
    function toJSON(group) {
        return {
            displayValue: DisplayValue.toJSON(group.displayValue),
            groupedRawValues: group.groupedRawValues.map(Value.toJSON),
        };
    }
    DisplayValueGroup.toJSON = toJSON;
    /** Deserialize [[DisplayValueGroup]] from JSON */
    function fromJSON(json) {
        return {
            displayValue: DisplayValue.fromJSON(json.displayValue),
            groupedRawValues: json.groupedRawValues.map(Value.fromJSON),
        };
    }
    DisplayValueGroup.fromJSON = fromJSON;
})(DisplayValueGroup = exports.DisplayValueGroup || (exports.DisplayValueGroup = {}));
function isNestedContentValue(v) {
    return (v !== undefined) && Array.isArray(v)
        && ((v.length === 0)
            || v[0].primaryKeys !== undefined
                && v[0].values !== undefined
                && v[0].displayValues !== undefined
                && v[0].mergedFieldNames !== undefined);
}
function isArrayValue(v) {
    // note: we don't guarantee by 100% that v is ValuesArray | DisplayValuesArray, but merely make compiler happy.
    // we have other means to determine the type of value.
    return (v !== undefined) && Array.isArray(v);
}
function isMapValue(v) {
    return (v !== undefined) && (typeof v === "object") && !Array.isArray(v);
}
function isPrimitiveValue(v) {
    return !isArrayValue(v) && !isMapValue(v);
}
function valuesArrayFromJSON(json) {
    return json.map(Value.fromJSON);
}
function valuesArrayToJSON(values) {
    return values.map(Value.toJSON);
}
function valuesMapFromJSON(json) {
    const map = {};
    for (const key in json) {
        /* istanbul ignore else */
        if (json.hasOwnProperty(key)) {
            map[key] = Value.fromJSON(json[key]);
        }
    }
    return map;
}
function valuesMapToJSON(values) {
    const map = {};
    for (const key in values) {
        /* istanbul ignore else */
        if (values.hasOwnProperty(key)) {
            map[key] = Value.toJSON(values[key]);
        }
    }
    return map;
}
function displayValuesArrayFromJSON(json) {
    return json.map(DisplayValue.fromJSON);
}
function displayValuesArrayToJSON(values) {
    return values.map(DisplayValue.toJSON);
}
function displayValuesMapFromJSON(json) {
    const map = {};
    for (const key in json) {
        /* istanbul ignore else */
        if (json.hasOwnProperty(key)) {
            map[key] = DisplayValue.fromJSON(json[key]);
        }
    }
    return map;
}
function displayValuesMapToJSON(values) {
    const map = {};
    for (const key in values) {
        /* istanbul ignore else */
        if (values.hasOwnProperty(key)) {
            map[key] = DisplayValue.toJSON(values[key]);
        }
    }
    return map;
}
//# sourceMappingURL=Value.js.map